/*
 * Copyright 2012-2017 Amazon Technologies, Inc.
 * 
 * Licensed under the Amazon Software License (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at:
 * 
 * http://aws.amazon.com/asl
 * 
 * This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES
 * OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and
 * limitations under the License.
 */ 

package com.amazonaws.mturk.cmd.test;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Appender;
import org.apache.log4j.FileAppender;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;
import junit.framework.TestCase;
import junit.textui.TestRunner;

import com.amazonaws.mturk.addon.HITDataCSVWriter;
import com.amazonaws.mturk.addon.HITDataOutput;
import com.amazonaws.mturk.addon.HITProperties;
import com.amazonaws.mturk.requester.HIT;
import com.amazonaws.mturk.requester.QualificationType;
import com.amazonaws.mturk.requester.QualificationTypeStatus;
import com.amazonaws.mturk.service.axis.RequesterService;
import com.amazonaws.mturk.service.exception.ServiceException;
import com.amazonaws.mturk.util.ClientConfig;
import com.amazonaws.mturk.util.PropertiesClientConfig;

public class TestBase extends TestCase {
  
  protected String testDir = "../etc/test";
  protected String inputFile = testDir + "/testInput.tab";
  protected String propertiesFile = testDir + "/testProperties.txt";
  protected String propertiesFile2 = testDir + "/testProperties2.txt";
  protected String qualPropsFile = testDir + "/testQualProps.txt";
  protected String questionFile = testDir + "/testQuestion.txt";
  protected String qualificationTestFile = testDir + "/testQualificationTest.xml";
  protected String answerKeyFile = testDir + "/testAnswerKey.xml";
  protected String previewFile = "testPreview.tmp"; 
  protected String testHITId;
  protected String testHITTypeId;
  protected String testQualTypeId;
  protected String unique;

  protected Appender logger;

  private static final String DEFAULT_LOG_FILE = "../log/aws-mturk-clt-test.log";

  private static RequesterService service = null; 
  
  static {
    ClientConfig config = new PropertiesClientConfig();
    config.setServiceURL(ClientConfig.SANDBOX_SERVICE_URL); 
    service = new RequesterService(config);    
  }
  
  public static void main(String[] args) {
    TestRunner.run(TestBase.class);
  }

  public TestBase(String testName) {
    super(testName);
  }
  
  Enumeration old_appenders;
  
  @Override
  protected void setUp() throws Exception {
    super.setUp();
    logger = new FileAppender(new PatternLayout("[%d{yyyy-MM-dd HH:mm:ss}] %-5p (%F: %L) - %m%n"), DEFAULT_LOG_FILE);
    unique = Long.toString(System.currentTimeMillis());
    old_appenders = Logger.getRootLogger().getAllAppenders();
    Logger.getRootLogger().removeAllAppenders();
    Logger.getRootLogger().addAppender(logger);
    FileWriter fw =  new FileWriter(new File(DEFAULT_LOG_FILE));
    fw.write("");
    fw.close();
  }

  @Override
  protected void tearDown() throws Exception {
    Logger.getRootLogger().removeAppender(logger);
    while( old_appenders.hasMoreElements() ) {
      Logger.getRootLogger().addAppender( (Appender) old_appenders.nextElement() );
    }
    super.tearDown();
  }

    protected String getTestHITId() throws ServiceException {
        if (testHITId == null) {

            HIT hit = createHIT();
            testHITId = hit.getHITId();
            testHITTypeId = hit.getHITTypeId();
        }

        return testHITId;
    }

    protected HIT createHIT() throws ServiceException {
      HIT hit = service.createHIT(null, // HITTypeId
          "SDK test " + unique,
          "test description", null, // keywords
          RequesterService.getBasicFreeTextQuestion("What's up?"), 0.00,
          (long) 3600, (long) 3600,
          (long) 3600, 1, null, // requesterAnnotation
          null, // qual requirements
          null  // responseGroup
      );

      assertNotNull(hit);
      assertNotNull(hit.getHITId());

      return hit;
    }
  
  protected String getTestHITTypeId() {
    if (testHITTypeId == null) {

      HIT hit = createHIT();
      testHITId = hit.getHITId();
      testHITTypeId = hit.getHITTypeId();
    }

    return testHITTypeId;
  }

    protected String getSuccessFile() throws ServiceException, IOException {
        String testHITId = getTestHITId();
        String testHITTypeId = getTestHITTypeId();

        String successFile = "test" + Long.toString(System.currentTimeMillis()) + ".success";
        String successFilePath = new File(successFile).getAbsolutePath();
        HITDataOutput writer = new HITDataCSVWriter(successFilePath, '\t', false);
        try {
            writer.setFieldNames(new String[] { HITProperties.HITField.HitId.getFieldName(),
                HITProperties.HITField.HitTypeId.getFieldName()});
            
            Map<String, String> values = new HashMap<String, String>(); 
            values.put(HITProperties.HITField.HitId.getFieldName(), testHITId);
            values.put(HITProperties.HITField.HitTypeId.getFieldName(), testHITTypeId);
            writer.writeValues(values);
        }
        finally {
            writer.close();
        }
        return successFilePath;
    }

    protected String getTestQualificationTypeId() throws ServiceException {
        if (testQualTypeId == null) {
            testQualTypeId = this.createQualificationType().getQualificationTypeId();
        }

        return testQualTypeId;
    }
  
    protected QualificationType createQualificationType() throws ServiceException {
        QualificationType qualType = service.createQualificationType(
                "cmd tool test qual type" + unique,
                null, // keywords
                "cmd tool test qual type description", QualificationTypeStatus.Active, (long) 0,
                    null, // test
                    null, // answerKey
                    null, // testDurationInSeconds
                    null, // autoGranted
                    null // autoGrantedValue
                    );

                assertNotNull(qualType);

                return qualType;
    }

    protected void deleteFile(String fileName) {
        File file = new File(fileName);
        if (file.exists()) {
            assertTrue("Failed to delete " + fileName, file.delete());
        }
    }

    protected void expectError(String error) throws IOException {
        FileReader logFile = new FileReader(new File(DEFAULT_LOG_FILE));
        BufferedReader br = new BufferedReader(logFile);
        assertTrue("Expected error <" + error + ">", fileContainsContent(br, error));
        br.close();
        logFile.close();
    }

    protected void expectNoError(String error) throws IOException {
        FileReader logFile = new FileReader(new File(DEFAULT_LOG_FILE));
        BufferedReader br = new BufferedReader(logFile);
        assertFalse("Didn't expect error <" + error + ">", fileContainsContent(br, error));
        br.close();
        logFile.close();
    }

    private boolean fileContainsContent(BufferedReader br, String content) throws IOException {
        for (String line = br.readLine(); line != null; line = br.readLine()) {
            if (line.contains(content)) {
                return true;
            }
        }
        return false;
    }

    protected void checkFile(String fileName, boolean shouldExist) {
        File f = new File(fileName);

        if (shouldExist) {
            assertTrue(f.exists());
        }
        else {
            assertFalse(f.exists());
        }
    }
}

